#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#ifdef USE_NLLIBC
#include <nllibc.h>
#endif

#include "args.h"

static char **expand_argv(int argc, char *argv[], int new_argc)
{
  char **new_argv;

  if ((argv != NULL) && (new_argc <= argc))
    return argv;

  new_argv = malloc(sizeof(char *) * (new_argc + 1));
  if (new_argv == NULL) {
    if (argv != NULL)
      free_argv(argv);
    return NULL;
  }

  if (argv != NULL) {
    memcpy(new_argv, argv, sizeof(char *) * argc);
    free(argv);
  }

  return new_argv;
}

int get_argc(char *argv[])
{
  int argc = 0;
  while (argv[argc])
    argc++;
  return argc;
}

char **make_argv(char *line)
{
  unsigned char *p, *arg, *word_start, *word_end;
  int argc = 0, len;
  char **argv;

  p = (unsigned char *)line;

  argv = expand_argv(0, NULL, 0);
  if (argv == NULL)
    return NULL;

  while (1) {
    while (*p && isspace(*p))
      p++;
    word_start = p;
    while (*p && !isspace(*p))
      p++;
    word_end = p;

    if (word_start == word_end)
      break;

    argv = expand_argv(argc, argv, argc + 1);
    if (argv == NULL)
      break;

    len = word_end - word_start;
    arg = malloc(len + 1);
    if (arg == NULL)
      break;
    memcpy(arg, word_start, len);
    arg[len] = '\0';

    argv[argc++] = (char *)arg;
  }

  argv[argc] = NULL;

  return argv;
}

char *free_argv(char *argv[])
{
  int i;

  if (argv) {
    for (i = 0; argv[i]; i++) {
      free(argv[i]);
    }
    free(argv);
  }

  return NULL;
}

int delete_argv(char *argv[], int argn)
{
  int argc, i;

  argc = get_argc(argv);

  for (i = 0; i < argn; i++) {
    free(argv[i]);
  }

  argc -= argn;

  for (i = 0; i < argc; i++) {
    argv[i] = argv[argn + i];
  }
  argv[i] = NULL;

  return argc;
}

char **split_argv(char *argv[], int argn)
{
  int argc, i;
  char **new_argv;

  argc = get_argc(argv);

  new_argv = malloc(sizeof(char *) * (argn + 1));
  if (new_argv == NULL)
    return NULL;

  for (i = 0; i < argn; i++) {
    new_argv[i] = argv[i];
  }
  new_argv[i] = NULL;

  argc -= argn;

  for (i = 0; i < argc; i++) {
    argv[i] = argv[argn + i];
  }
  argv[i] = NULL;

  return new_argv;
}
